#!/usr/bin/env python 3
from olctools.accessoryFunctions.accessoryFunctions import printtime
from rauth import OAuth1Session
import multiprocessing
import os
import re

"""
Script to test access to authenticated resources via REST interface.
Written by Keith Jolley
Copyright (c) 2017, University of Oxford
E-mail: keith.jolley@zoo.ox.ac.uk

This file is part of Bacterial Isolate Genome Sequence Database (BIGSdb).

BIGSdb is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

BIGSdb is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

The test databases can be reached at https://pubmlst.org/test/.
To use these, sign up for a PubMLST account (https://pubmlst.org/site_accounts.shtml)
and link this account with the pubmlst_test_seqdef and pubmlst_test_isolates 
databases (https://pubmlst.org/site_accounts.shtml#registering_with_databases)
"""


'modified by adamkoziol'


class REST(object):

    def main(self):
        """
        Run the appropriate methods in the correct order
        """
        self.secret_finder()
        self.parse_access_token()
        self.get_session_token()
        self.parse_session_token()
        self.get_route()
        self.download_profile()
        self.find_loci()
        self.download_loci()

    def secret_finder(self):
        """
        Parses the supplied secret.txt file for the consumer key and secrets
        """
        secretlist = list()
        if os.path.isfile(self.secret_file):
            # Open the file, and put the contents into a list
            with open(self.secret_file, 'r') as secret:
                for line in secret:
                    secretlist.append(line.rstrip())
            # Extract the key and secret from the list
            self.consumer_key = secretlist[0]
            self.consumer_secret = secretlist[1]
        else:
            print('"Cannot find the secret.txt file required for authorization. Please ensure that this file exists, '
                  'and that the supplied consumer key is on the first line, and the consumer secret is on he second '
                  'line. Contact keith.jolley@zoo.ox.ac.uk for an account, and the necessary keys')
            quit()

    def parse_access_token(self):
        """
        Extract the secret and token values from the access_token file
        """
        access_file = os.path.join(self.file_path, 'access_token')
        # Ensure that the access_token file exists
        if os.path.isfile(access_file):
            # Initialise a list to store the secret and token
            access_list = list()
            with open(access_file, 'r') as access_token:
                for line in access_token:
                    value, data = line.split('=')
                    access_list.append(data.rstrip())
            # Set the variables appropriately
            self.access_secret = access_list[0]
            self.access_token = access_list[1]
        else:
            print('Missing access_token')

    def get_session_token(self):
        """
        Use the accession token to request a new session token
        """
        printtime('Getting session token', self.start)
        # Rather than testing any previous session tokens to see if they are still valid, simply delete old tokens in
        # preparation of the creation of new ones
        try:
            os.remove(os.path.join(self.file_path, 'session_token'))
        except FileNotFoundError:
            pass
        # Create a new session
        session_request = OAuth1Session(self.consumer_key,
                                        self.consumer_secret,
                                        access_token=self.access_token,
                                        access_token_secret=self.access_secret)
        # Set the URL appropriately
        url = self.test_rest_url + '/oauth/get_session_token'
        # Perform a GET request with the appropriate keys and tokens
        r = session_request.get(url)
        # If the status code is '200' (OK), proceed
        if r.status_code == 200:
            # Save the JSON-decoded token secret and token
            self.session_token = r.json()['oauth_token']
            self.session_secret = r.json()['oauth_token_secret']
            # Write the token and secret to file
            self.write_token('session_token', self.session_token, self.session_secret)
        # Any other status than 200 is considered a failure
        else:
            print('Failed:')
            print(r.json()['message'])

    def write_token(self, token_type, token, secret):
        """
        Write a token to file. Format is secret='secret'\,token='token'
        :param token_type: The type of token. Currently only 'session' is used, but 'access' should also be an option
        :param token: The string of the token extracted from the GET request
        :param secret:
        """
        # Open the file, and write the token and secret strings appropriately
        with open(os.path.join(self.file_path, token_type), 'w') as token_file:
            token_file.write('secret=' + secret + '\n')
            token_file.write('token=' + token + '\n')

    def parse_session_token(self):
        """
        Extract the session secret and token strings from the session token file
        """
        session_file = os.path.join(self.file_path, 'session_token')
        # Only try to extract the strings if the file exists
        if os.path.isfile(session_file):
            # Create a list to store the data from the file
            session_list = list()
            with open(session_file, 'r') as session_token:
                for line in session_token:
                    # Split the description e.g. secret= from the line
                    value, data = line.split('=')
                    # Add each string to the list
                    session_list.append(data.rstrip())
            # Extract the appropriate variable from the list
            self.session_secret = session_list[0]
            self.session_token = session_list[1]

    def get_route(self):
        """
        Creates a session to find the URL for the loci and schemes
        """
        # Create a new session
        session = OAuth1Session(self.consumer_key,
                                self.consumer_secret,
                                access_token=self.session_token,
                                access_token_secret=self.session_secret)
        # Use the test URL in the GET request
        r = session.get(self.test_rest_url)
        if r.status_code == 200 or r.status_code == 201:
            if re.search('json', r.headers['content-type'], flags=0):
                decoded = r.json()
            else:
                decoded = r.text
            # Extract the URLs from the returned data
            self.loci = decoded['loci']
            self.profile = decoded['schemes']

    def download_profile(self):
        """
        Download the profile from the database
        """
        printtime('Downloading profile', self.start)
        # Set the name of the profile file
        profile_file = os.path.join(self.output_path, 'profile.txt')
        size = 0
        # Ensure that the file exists, and that it is not too small; likely indicating a failed download
        try:
            stats = os.stat(profile_file)
            size = stats.st_size
        except FileNotFoundError:
            pass
        # Only download the profile if the file doesn't exist, or is likely truncated
        if not os.path.isfile(profile_file) or size <= 100:
            # Create a new session
            session = OAuth1Session(self.consumer_key,
                                    self.consumer_secret,
                                    access_token=self.session_token,
                                    access_token_secret=self.session_secret)
            # The profile file is called profiles_csv on the server. Updated the URL appropriately
            r = session.get(self.profile + '/1/profiles_csv')
            # On a successful GET request, parse the returned data appropriately
            if r.status_code == 200 or r.status_code == 201:
                if re.search('json', r.headers['content-type'], flags=0):
                    decoded = r.json()
                else:
                    decoded = r.text
                # Write the profile file to disk
                with open(profile_file, 'w') as profile:
                    profile.write(decoded)

    def find_loci(self):
        """
        Finds the URLs for all allele files
        """
        printtime('Downloading alleles', self.start)
        session = OAuth1Session(self.consumer_key,
                                self.consumer_secret,
                                access_token=self.session_token,
                                access_token_secret=self.session_secret)
        # Use the URL for all loci determined above
        r = session.get(self.loci)
        if r.status_code == 200 or r.status_code == 201:
            if re.search('json', r.headers['content-type'], flags=0):
                decoded = r.json()
            else:
                decoded = r.text
            # Extract all the URLs in the decoded dictionary under the key 'loci'
            for locus in decoded['loci']:
                # Add each URL to the list
                self.loci_url.append(locus)

    def download_loci(self):
        """
        Uses a multi-threaded approach to download allele files
        """
        # Setup the multiprocessing pool.
        pool = multiprocessing.Pool(processes=self.threads)
        # Map the list of loci URLs to the download method
        pool.map(self.download_threads, self.loci_url)
        pool.close()
        pool.join()

    def download_threads(self, url):
        """
        Download the allele files
        """
        # Set the name of the allele file - split the gene name from the URL
        output_file = os.path.join(self.output_path, '{}.tfa'.format(os.path.split(url)[-1]))
        # Check to see whether the file already exists, and if it is unusually small
        size = 0
        try:
            stats = os.stat(output_file)
            size = stats.st_size
        except FileNotFoundError:
            pass
        # If the file doesn't exist, or is truncated, proceed with the download
        if not os.path.isfile(output_file) or size <= 100:
            # Create a new session
            session = OAuth1Session(self.consumer_key,
                                    self.consumer_secret,
                                    access_token=self.session_token,
                                    access_token_secret=self.session_secret)
            # The allele file on the server is called alleles_fasta. Update the URL appropriately
            r = session.get(url + '/alleles_fasta')
            if r.status_code == 200 or r.status_code == 201:
                if re.search('json', r.headers['content-type'], flags=0):
                    decoded = r.json()
                else:
                    decoded = r.text
                # Write the allele to disk
                with open(output_file, 'w') as allele:
                    allele.write(decoded)

    def __init__(self, args):
        self.test_rest_url = 'http://rest.pubmlst.org/db/pubmlst_rmlst_seqdef'
        self.test_web_url = 'http://pubmlst.org/cgi-bin/bigsdb/bigsdb.pl?db=pubmlst_rmlst_seqdef'
        self.request_token_url = self.test_rest_url + '/oauth/get_request_token'
        self.access_token_url = self.test_rest_url + '/oauth/get_access_token'
        self.authorize_url = self.test_web_url + '&page=authorizeClient'
        self.secret_file = args.secret_file
        self.file_path = args.file_path
        self.output_path = args.output_path
        self.start = args.start
        self.consumer_key = str()
        self.consumer_secret = str()
        self.access_secret = str()
        self.access_token = str()
        self.session_secret = str()
        self.session_token = str()
        self.loci = str()
        self.profile = str()
        self.loci_url = list()
        self.threads = multiprocessing.cpu_count()



#!/usr/bin/env python
import os


def test_imports():
    """
    Test the imports
    """
    for root, dirs, files in os.walk('.'):
        for f in files:
            package = '.'.join(os.path.split(root.lstrip('./')))
            module = os.path.splitext(f)[0]
            if f.endswith('.py') and '__' not in f and '__' not in root and 'test_' not in f and root != '.' \
                    and f != 'setup.py':
                if 'cowbat' not in package:
                    import_statement = 'import {package}.{module}'.format(package=package,
                                                                          module=module)
                    print(import_statement.rstrip())
                    exec(import_statement.rstrip())
                # Try all the imports from within the file
                with open(os.path.join(root, f), 'r') as python_file:
                    for line in python_file:
                        if line.startswith('from') or line.startswith('import'):
                            statement = line.rstrip()
                            if len(statement.split(',')) == 1:
                                print(line.rstrip())
                                exec(line.rstrip())
                            else:
                                # from accessoryFunctions.accessoryFunctions import make_path, MetadataObject
                                if line.startswith('from'):
                                    if line.rstrip().endswith('\\'):
                                        line = line.rstrip().replace('\\', '') + next(python_file).rstrip().lstrip()
                                    import_base, import_line = line.split(' import ')
                                    imported_packages = import_line.split(',')
                                    for imported_package in imported_packages:
                                        # Don't check relative imports
                                        if '.' not in import_base:
                                            import_string = '{import_base} import {import_package}' \
                                                .format(import_base=import_base,
                                                        import_package=imported_package)
                                            print(import_string)
                                            exec(import_string)
                                # import sys, os
                                else:
                                    imported_packages = line.lstrip('import').split(', ')
                                    for imported_package in imported_packages:
                                        import_string = 'import {imp_package}' \
                                            .format(imp_package=imported_package.lstrip().rstrip())
                                        print(import_string)
                                        exec(import_string)



#!/usr/bin/env python 3
from olctools.accessoryFunctions.accessoryFunctions import printtime, make_path
from cowbat.get import rest_auth_class
from Bio import SeqIO
from argparse import ArgumentParser
from glob import glob
import time
import os


__author__ = 'adamkoziol'


class Get(object):

    def getrmlsthelper(self):
        """
        Makes a system call to rest_auth.py, a Python script modified from
        https://github.com/kjolley/BIGSdb/tree/develop/scripts/test
        And downloads the most up-to-date rMLST profile and alleles
        """

        printtime('Downloading {} alleles'.format(self.analysistype), self.start)
        # Extract the path of the current script from the full path + file name
        homepath = os.path.split(os.path.abspath(__file__))[0]
        # Set the path/name of the folder to contain the new alleles and profile
        newfolder = os.path.join(self.path, self.analysistype)
        # Create the path
        make_path(newfolder)
        # Create arguments to feed into the rest_auth_class script
        args = ArgumentParser
        args.secret_file = os.path.join(homepath, 'secret.txt')
        args.file_path = homepath
        args.output_path = newfolder
        args.start = self.start
        rmlst = rest_auth_class.REST(args)
        # Download the profile and alleles
        rmlst.main()

        # Get the new alleles into a list, and create the combinedAlleles file
        alleles = glob(os.path.join(newfolder, '*.tfa'))
        self.combinealleles(newfolder, alleles)

    def combinealleles(self, allelepath, alleles):
        printtime('Creating combined rMLST allele file', self.start)
        with open(os.path.join(allelepath, 'rMLST_combined.fasta'), 'w') as combinedfile:
            # Open each allele file
            for allele in sorted(alleles):
                # with open(allele, 'rU') as fasta:
                for record in SeqIO.parse(open(allele, "rU"), "fasta"):
                    # Extract the sequence record from each entry in the multifasta
                    # Replace and dashes in the record.id with underscores
                    record.id = record.id.replace('-', '_')
                    # Remove and dashes or 'N's from the sequence data - makeblastdb can't handle sequences
                    # with gaps
                    # noinspection PyProtectedMember
                    record.seq._data = record.seq._data.replace('-', '').replace('N', '')
                    # Clear the name and description attributes of the record
                    record.name = ''
                    record.description = ''
                    # Write each record to the combined file
                    SeqIO.write(record, combinedfile, 'fasta')

    def __init__(self, args):
        self.path = os.path.join(args.path)
        self.start = args.start
        self.analysistype = 'rMLST'
        self.getrmlsthelper()


if __name__ == '__main__':
    # Argument parser for user-inputted values, and a nifty help menu
    # Parser for arguments
    parser = ArgumentParser(description='')
    parser.add_argument('path',
                        help='Specify input directory')

    # Get the arguments into an object
    arguments = parser.parse_args()

    # Define the start time
    arguments.start = time.time()

    # Run the script
    Get(arguments)

    # Print a bold, green exit statement
    print('\033[92m' + '\033[1m' + "\nElapsed Time: %0.2f seconds" % (time.time() - arguments.start) + '\033[0m')



#!/usr/bin/env python3
from olctools.accessoryFunctions.accessoryFunctions import GenObject, MetadataObject, make_path, SetupLogging
import olctools.accessoryFunctions.metadataprinter as metadataprinter
from genemethods.assemblypipeline import fileprep, createobject
from cowbat.metagenomefilter import filtermetagenome
from argparse import ArgumentParser
from shutil import move, which
from click import progressbar
from threading import Thread
from csv import DictReader
from queue import Queue
import subprocess
import xlsxwriter
import logging
import time
import os
__author__ = 'adamkoziol'


class CLARK(object):

    def main(self):
        """

        """
        # Set the targets
        self.settargets()
        # Classify the metagenome(s)
        if self.clean_seqs:
            self.clean_sequences()
        self.lists()
        self.classifymetagenome()
        # Estimate the abundance
        self.estimateabundance()
        # Create reports
        self.reports()

    def objectprep(self):
        """Create objects to store data and metadata for each sample. Also, perform necessary file manipulations"""
        # Move the files to subfolders and create objects
        self.runmetadata = createobject.ObjectCreation(self)
        if self.extension == 'fastq':
            # To streamline the CLARK process, decompress and combine .gz and paired end files as required
            logging.info('Decompressing and combining .fastq files for CLARK analysis')
            fileprep.Fileprep(self)
        else:
            logging.info('Using .fasta files for CLARK analysis')
            for sample in self.runmetadata.samples:
                sample.general.combined = sample.general.fastqfiles[0]

    def settargets(self):
        """Set the targets to be used in the analyses. Involves the path of the database files, the database files to
         use, and the level of classification for the analysis"""
        # Define the set targets call. Include the path to the script, the database path and files, as well
        # as the taxonomic rank to use
        logging.info('Setting up database')
        self.targetcall = 'cd {} && ./set_targets.sh {} {} --{}'.format(self.clarkpath, self.databasepath,
                                                                        self.database, self.rank)
        #
        subprocess.call(self.targetcall, shell=True, stdout=self.devnull, stderr=self.devnull)

    def clean_sequences(self):
        """Removes reads/contigs that contain plasmids, and masks phage sequences."""
        logging.info('Removing plasmids and masking phages')
        plasmid_db = os.path.join(self.reffilepath, 'plasmidfinder', 'plasmid_database.fa')
        phage_db = os.path.join(self.reffilepath, 'prophages', 'combinedtargets.tfa')
        with progressbar(self.runmetadata.samples) as bar:
            for sample in bar:
                plasmid_removal = 'bbduk.sh ref={} in={} out={} overwrite'\
                    .format(plasmid_db, sample.general.combined, sample.general.combined.replace('.f', '_noplasmid.f'))
                subprocess.call(plasmid_removal, shell=True, stdout=self.devnull, stderr=self.devnull)
                phage_masking = 'bbduk.sh ref={} in={} out={} kmask=N overwrite'\
                    .format(phage_db, sample.general.combined.replace('.f', '_noplasmid.f'),
                            sample.general.combined.replace('.f', '_clean.f'))
                subprocess.call(phage_masking, shell=True, stdout=self.devnull, stderr=self.devnull)
                os.remove(sample.general.combined)
                os.rename(sample.general.combined.replace('.f', '_clean.f'), sample.general.combined)
                os.remove(sample.general.combined.replace('.f', '_noplasmid.f'))

    def classifymetagenome(self):
        """Run the classify metagenome of the CLARK package on the samples"""
        logging.info('Classifying metagenomes')
        # Define the system call
        self.classifycall = 'cd {} && ./classify_metagenome.sh -O {} -R {} -n {} --light'\
            .format(self.clarkpath,
                    self.filelist,
                    self.reportlist,
                    self.cpus)
        # Variable to store classification state
        classify = True
        for sample in self.runmetadata.samples:
            try:
                # Define the name of the .csv classification file
                sample.general.classification = sample.general.combined.split('.')[0] + '.csv'
                # If the file exists, then set classify to False
                if os.path.isfile(sample.general.classification):
                    classify = False
            except KeyError:
                pass
        # Run the system call if the samples have not been classified
        if classify:
            # Run the call
            subprocess.call(self.classifycall, shell=True, stdout=self.devnull, stderr=self.devnull)

    def lists(self):
        """
        Prepare the list of files to be processed
        """
        # Prepare the lists to be used to classify the metagenomes
        with open(self.filelist, 'w') as filelist:
            with open(self.reportlist, 'w') as reportlist:
                for sample in self.runmetadata.samples:
                    if self.extension == 'fastq':
                        try:
                            status = sample.run.Description
                            if status == 'metagenome':
                                filelist.write(sample.general.combined + '\n')
                                reportlist.write(sample.general.combined.split('.')[0] + '\n')
                        except AttributeError:
                            pass
                    else:
                        if sample.general.combined != 'NA':
                            filelist.write(sample.general.combined + '\n')
                            reportlist.write(sample.general.combined.split('.')[0] + '\n')

    def estimateabundance(self):
        """
        Estimate the abundance of taxonomic groups
        """
        logging.info('Estimating abundance of taxonomic groups')
        # Create and start threads
        for i in range(self.cpus):
            # Send the threads to the appropriate destination function
            threads = Thread(target=self.estimate, args=())
            # Set the daemon to true - something to do with thread management
            threads.setDaemon(True)
            # Start the threading
            threads.start()
        with progressbar(self.runmetadata.samples) as bar:
            for sample in bar:
                try:
                    if sample.general.combined != 'NA':
                        # Set the name of the abundance report
                        sample.general.abundance = sample.general.combined.split('.')[0] + '_abundance.csv'
                        # if not hasattr(sample, 'commands'):
                        if not sample.commands.datastore:
                            sample.commands = GenObject()

                        # Define system calls
                        sample.commands.target = self.targetcall
                        sample.commands.classify = self.classifycall
                        sample.commands.abundancecall = \
                            'cd {} && ./estimate_abundance.sh -D {} -F {} > {}'.format(self.clarkpath,
                                                                                       self.databasepath,
                                                                                       sample.general.classification,
                                                                                       sample.general.abundance)
                        self.abundancequeue.put(sample)
                except KeyError:
                    pass
        self.abundancequeue.join()

    def estimate(self):
        while True:
            sample = self.abundancequeue.get()

            # Run the system call (if necessary)
            if not os.path.isfile(sample.general.abundance):
                #
                subprocess.call(sample.commands.abundancecall, shell=True, stdout=self.devnull, stderr=self.devnull)
            self.abundancequeue.task_done()

    def reports(self):
        """
        Create reports from the abundance estimation
        """
        logging.info('Creating CLARK report for {ft} files'.format(ft=self.extension))
        # Create a workbook to store the report. Using xlsxwriter rather than a simple csv format, as I want to be
        # able to have appropriately sized, multi-line cells
        workbook = xlsxwriter.Workbook(self.report)
        make_path(self.reportpath)
        # New worksheet to store the data
        worksheet = workbook.add_worksheet()
        # Add a bold format for header cells. Using a monotype font size 8
        bold = workbook.add_format({'bold': True, 'font_name': 'Courier New', 'font_size': 8})
        bold.set_align('center')
        # Format for data cells. Monotype, size 8, top vertically justified
        courier = workbook.add_format({'font_name': 'Courier New', 'font_size': 8})
        courier.set_align('top')
        # Set the custom width for 5 and 6 to be 15
        worksheet.set_column(5, 5, 15)
        worksheet.set_column(6, 6, 20)
        # Initialise the position within the worksheet to be (0,0)
        row = 0
        col = 0
        # List of the headers to use
        headers = ['Strain', 'Name', 'TaxID', 'Lineage', 'Count', 'Proportion_All(%)', 'Proportion_Classified(%)']
        # Add an additional header for .fasta analyses
        if self.extension == 'fasta':
            headers.insert(4, 'TotalBP')
        # Populate the headers
        for category in headers:
            # Write the data in the specified cell (row, col) using the bold format
            worksheet.write(row, col, category, bold)
            # Move to the next column to write the next category
            col += 1
        # Data starts in row 1
        row = 1
        # Initialise variables to hold the longest names; used in setting the column width
        longeststrain = 0
        longestname = 0
        longestlineage = 0
        # Extract all the taxonomic groups that pass the cutoff from the abundance file
        for sample in self.runmetadata.samples:
            # Every record starts at column 0
            col = 0
            # Write the strain name
            worksheet.write(row, col, sample.name, courier)
            col += 1
            # Initialise a dictionary to store the species above the cutoff in the sample
            sample.general.passfilter = list()
            try:
                # Abundance file as a dictionary
                abundancedict = DictReader(open(sample.general.abundance))
                # Filter abundance to taxIDs with at least self.cutoff% of the total proportion
                for result in abundancedict:
                    # The UNKNOWN category doesn't contain a 'Lineage' column, and therefore, subsequent columns are
                    # shifted out of proper alignment, and do not contain the appropriate data
                    try:
                        if float(result['Proportion_All(%)']) > self.cutoff:
                            sample.general.passfilter.append(result)
                    except ValueError:
                        pass
                # Determine the longest name of all the strains, and use it to set the width of column 0
                if len(sample.name) > longeststrain:
                    longeststrain = len(sample.name)
                    worksheet.set_column(0, 0, longeststrain)
                # Sort the abundance results based on the highest count
                sortedabundance = sorted(sample.general.passfilter, key=lambda x: int(x['Count']), reverse=True)
                # Set of contigs from the classification file. For some reason, certain contigs are represented multiple
                # times in the classification file. As far as I can tell, these multiple representations are always
                # classified the same, and, therefore, should be treated as duplicates, and ignored
                contigset = set()
                for result in sortedabundance:
                    # Add the total number of base pairs classified for each TaxID. As only the total number of contigs
                    # classified as a particular TaxID are in the report, it can be misleading if a large number
                    # of small contigs are classified to a particular TaxID e.g. 56 contigs map to TaxID 28901, and 50
                    # contigs map to TaxID 630, however, added together, those 56 contigs are 4705838 bp, while the 50
                    # contigs added together are only 69602 bp. While this is unlikely a pure culture, only
                    # 69602 / (4705838 + 69602) = 1.5% of the total bp map to TaxID 630 compared to 45% of the contigs
                    if self.extension == 'fasta':
                        # Initialise a variable to store the total bp mapped to the TaxID
                        result['TotalBP'] = int()
                        # Read the classification file into a dictionary
                        classificationdict = DictReader(open(sample.general.classification))
                        # Read through each contig classification in the dictionary
                        for contig in classificationdict:
                            # Pull out each contig with a TaxID that matches the TaxID of the result of interest, and
                            # is not present in a set of contigs that have already been added to the dictionary
                            if result['TaxID'] == contig[' Assignment'] and contig['Object_ID'] not in contigset:
                                # Increment the total bp mapping to the TaxID by the integer of each contig
                                result['TotalBP'] += int(contig[' Length'])
                                # Avoid duplicates by adding the contig name to the set of contigs
                                contigset.add(contig['Object_ID'])
                    # Print the results to file
                    # Ignore the first header, as it is the strain name, which has already been added to the report
                    dictionaryheaders = headers[1:]
                    for header in dictionaryheaders:
                        data = result[header]
                        worksheet.write(row, col, data, courier)
                        col += 1
                        # Determine the longest name of all the matches, and use it to set the width of column 0
                        if len(result['Name']) > longestname:
                            longestname = len(result['Name'])
                            worksheet.set_column(1, 1, longestname)
                        # Do the same for the lineages
                        if len(result['Lineage']) > longestlineage:
                            longestlineage = len(result['Lineage'])
                            worksheet.set_column(3, 3, longestlineage)
                    # Increase the row
                    row += 1
                    # Set the column to 1
                    col = 1
            except (KeyError, AttributeError):
                # Increase the row
                row += 1
        # Close the workbook
        workbook.close()

    def __init__(self, args, pipelinecommit, startingtime, scriptpath):
        # Initialise variables
        self.commit = str(pipelinecommit)
        self.start = startingtime
        self.homepath = scriptpath
        # Define variables based on supplied arguments
        self.args = args
        self.path = os.path.join(args.path)
        assert os.path.isdir(self.path), u'Supplied path is not a valid directory {0!r:s}'.format(self.path)
        self.sequencepath = os.path.join(args.sequencepath, '')
        assert os.path.isdir(self.sequencepath), u'Supplied sequence path is not a valid directory {0!r:s}' \
            .format(self.sequencepath)
        self.databasepath = os.path.join(args.databasepath, '')
        assert os.path.isdir(self.databasepath), u'Supplied database path is not a valid directory {0!r:s}' \
            .format(self.databasepath)
        # There seems to be an issue with CLARK when running with a very high number of cores. Limit self.cpus to 1
        self.cpus = 4
        # Set variables from the arguments
        self.database = args.database
        self.rank = args.rank
        self.clarkpath = args.clarkpath
        self.cutoff = float(args.cutoff) * 100
        # Initialise variables for the analysis
        self.targetcall = str()
        self.classifycall = str()
        self.devnull = open(os.devnull, 'wb')
        self.filelist = os.path.join(self.path, 'sampleList.txt')
        self.reportlist = os.path.join(self.path, 'reportList.txt')
        self.abundancequeue = Queue()
        self.datapath = str()
        self.reportpath = os.path.join(self.path, 'reports')
        self.clean_seqs = args.clean_seqs
        self.light = args.light
        self.extension = args.extension
        if self.clean_seqs:
            try:
                self.reffilepath = args.reffilepath
            except AttributeError:
                self.clean_seqs = False
        # If run as part of the assembly pipeline, a few modifications are necessary to ensure that the metadata objects
        # and variables play nice
        try:
            if args.runmetadata:
                self.runmetadata = args.runmetadata
                # Create the name of the final report
                self.report = os.path.join(self.reportpath, 'abundance_{ft}.xlsx'.format(ft=self.extension))
                # Only re-run the CLARK analyses if the CLARK report doesn't exist. All files created by CLARK
                if not os.path.isfile(self.report):
                    logging.info('Performing CLARK analysis on {ft} files'.format(ft=self.extension))
                    if self.extension != 'fastq':
                        for sample in self.runmetadata.samples:
                            sample.general.combined = sample.general.bestassemblyfile
                        # Run the pipeline
                        self.main()
                    else:
                        # Only perform FASTQ analyses if the sample is declared to be a metagenome
                        metagenome = False
                        for sample in self.runmetadata.samples:
                            try:
                                status = sample.run.Description
                            except AttributeError:
                                status = 'unknown'
                            if status == 'metagenome':
                                metagenome = True
                        # If any of the samples are metagenomes, run the CLARK analysis on the raw files
                        if metagenome:
                            fileprep.Fileprep(self)
                            # Run the pipeline
                            self.main()
                    # Clean up the files and create/delete attributes to be consistent with pipeline Metadata objects
                    for sample in self.runmetadata.samples:
                        # Create a GenObject to store metadata when this script is run as part of the pipeline
                        clarkextension = 'clark{}'.format(self.extension)
                        setattr(sample, clarkextension, GenObject())
                        # Create a folder to store all the CLARK files
                        sample[clarkextension].outputpath = os.path.join(sample.general.outputdirectory, 'CLARK')
                        make_path(sample[clarkextension].outputpath)
                        if sample.general.bestassemblyfile != 'NA':
                            # Move the files to the CLARK folder
                            try:
                                move(sample.general.abundance,
                                     os.path.join(sample[clarkextension].outputpath,
                                                  os.path.basename(sample.general.abundance)))
                                move(sample.general.classification,
                                     os.path.join(sample[clarkextension].outputpath,
                                                  os.path.basename(sample.general.classification)))
                            except (AttributeError, FileNotFoundError):
                                pass
                            # Set the CLARK-specific attributes
                            try:
                                sample[clarkextension].abundance = sample.general.abundance
                                sample[clarkextension].classification = sample.general.classification
                                sample[clarkextension].combined = sample.general.combined
                            except AttributeError:
                                pass
                            if self.extension == 'fastq':
                                # Remove the combined .fastq files
                                try:
                                    if type(sample[clarkextension].combined) is list:
                                        os.remove(sample[clarkextension].combined)
                                except (OSError, AttributeError):
                                    pass
                        # Remove the text files lists of files and reports created by CLARK
                        try:
                            map(lambda x: os.remove(os.path.join(self.path, x)), ['reportList.txt', 'sampleList.txt'])
                        except OSError:
                            pass
            else:
                self.runmetadata = MetadataObject()
                self.report = os.path.join(self.reportpath, 'abundance.xlsx')
                # Create the objects
                self.objectprep()
                self.main()
        except AttributeError:
            self.runmetadata = MetadataObject()
            self.report = os.path.join(self.reportpath, 'abundance.xlsx')
            # Create the objects
            self.objectprep()
            # Set the run description to 'metagenome' in order to process the samples
            for sample in self.runmetadata.samples:
                sample.run.Description = 'metagenome'
            self.main()
        # Optionally filter the .fastq reads based on taxonomic assignment
        if args.filter:
            filtermetagenome.PipelineInit(self)
        # Print the metadata to file
        metadataprinter.MetadataPrinter(self)


if __name__ == '__main__':
    # Argument parser for user-inputted values, and a nifty help menu
    # Get the current commit of the pipeline from git
    # Extract the path of the current script from the full path + file name
    homepath = os.path.split(os.path.abspath(__file__))[0]
    # Find the commit of the script by running a command to change to the directory containing the script and run
    # a git command to return the final tag
    commit = subprocess.Popen('cd {} && git tag | tail -n 1'.format(homepath),
                              shell=True, stdout=subprocess.PIPE).communicate()[0].rstrip()
    # Parser for arguments
    parser = ArgumentParser(description='Automates CLARK metagenome software')
    parser.add_argument('path',
                        help='Specify input directory')
    parser.add_argument('-s', '--sequencepath',
                        required=True,
                        help='Path of .fastq(.gz) files to process.')
    parser.add_argument('-d', '--databasepath',
                        required=True,
                        help='Path of CLARK database files to use.')
    parser.add_argument('-C', '--clarkpath',
                        required=True,
                        help='Path to the CLARK scripts')
    parser.add_argument('-r', '--rank',
                        default='species',
                        help='Choose the taxonomic rank to use in the analysis: species (the default value), genus, '
                             'family, order, class or phylum')
    parser.add_argument('-D', '--database',
                        default='bacteria',
                        help='Choose the database to use in the analysis (one or more from: bacteria, viruses, human,'
                             'and custom. To select more than one, use commas to separate your selection. Custom'
                             'databases need to be set up before they will work')
    parser.add_argument('-t', '--threads',
                        help='Number of threads. Default is the number of cores in the system')
    parser.add_argument('-f', '--filter',
                        action='store_true',
                        help='Optionally split the samples based on taxonomic assignment')
    parser.add_argument('-c', '--cutoff',
                        default=0.01,
                        help='Cutoff value for setting taxIDs to use when filtering fastq files. Defaults to 1 percent.'
                             ' Please note that you must use a decimal format: enter 0.05 to get a 5 percent cutoff.')
    parser.add_argument('-cl', '--clean_seqs',
                        default=False,
                        action='store_true',
                        help='If enabled, removes plasmid sequences and masks phage sequences. Only usable if you '
                             'have access to the OLC NAS.')
    parser.add_argument('-l', '--light',
                        default=True,
                        help='Run CLARK in light mode for systems with lower RAM')
    parser.add_argument('-e', '--extension',
                        default='fastq',
                        help='Extension of file type to process. Must be either "fasta" or "fastq". Default is "fastq"')
    # Get the arguments into an object
    arguments = parser.parse_args()
    SetupLogging()
    # TODO Implement --rank and --database, and provide a way to easily create a custom database

    # Define the start time
    start = time.time()

    # Run the script
    CLARK(arguments, commit, start, homepath)

    # Print a bold, green exit statement
    logging.info('Analyses complete')


class PipelineInit(object):

    def __init__(self, inputobject, extension='fasta', light=False):
        # Create an object to mimic the command line arguments necessary for the script
        args = MetadataObject()
        args.path = inputobject.path
        args.sequencepath = inputobject.path
        args.databasepath = os.path.join(inputobject.reffilepath, 'clark')
        make_path(args.databasepath)
        args.clarkpath = os.path.dirname(which('CLARK'))
        args.clarkpath += '/../opt/clark/'
        args.cutoff = 0.005
        args.database = 'bacteria'
        args.rank = 'species'
        args.filter = False
        args.threads = inputobject.cpus
        args.runmetadata = inputobject.runmetadata
        args.clean_seqs = False
        args.reffilepath = inputobject.reffilepath
        args.extension = extension
        args.light = light
        # Run CLARK
        CLARK(args, inputobject.commit, inputobject.starttime, inputobject.homepath)



#!/usr/bin/env python
from setuptools import setup, find_packages
from distutils.util import convert_path
import os
__author__ = 'adamkoziol'

# Find the version
version = dict()
with open(convert_path(os.path.join('cowbat', 'version.py')), 'r') as version_file:
    exec(version_file.read(), version)

setup(
    name="COWBAT",
    version=version['__version__'],
    include_package_data=True,
    packages=find_packages(),
    scripts=[os.path.join('cowbat', 'assembly_pipeline.py'),
             os.path.join('cowbat', 'assembly_typing.py'),
             os.path.join('cowbat', 'validation', 'validate_cowbat.py')
             ],
    license='MIT',
    author='Adam Koziol',
    author_email='adam.koziol@inspection.gc.ca',
    description='CFIA OLC Workflow for Bacterial Assembly and Typing',
    url='https://github.com/OLC-Bioinformatics/COWBAT',
    long_description=open('README.md').read(),
)



#!/usr/bin/env python
from olctools.accessoryFunctions.accessoryFunctions import SetupLogging
from validator_helper import validate
from argparse import ArgumentParser
import logging
import os


class ValidateCowbat(object):

    def validate_cowbat(self):
        # combinedMetadata
        logging.info('Validating combinedMetadata.csv')
        self.validate_combined_metadata(reference_report=os.path.join(self.reference_folder, 'combinedMetadata.csv'),
                                        test_report=os.path.join(self.test_folder, 'combinedMetadata.csv'),
                                        assembly_typer=self.assembly_typer)
        logging.info('Double validating combinedMetadata.csv')
        self.validate_report(reference_report=os.path.join(self.reference_folder, 'combinedMetadata.csv'),
                             test_report=os.path.join(self.test_folder, 'combinedMetadata.csv'),
                             columns_to_exclude=['SeqID', 'AssemblyDate', 'MASH_NumMatchingHashes', 'CoreGenesPresent',
                                                 '16S_result', 'PlasmidProfile', 'PipelineVersion'],
                             identifying_column='SeqID')
        # AMRSummary
        logging.info('Validating amr_summary.csv')
        self.validate_report(reference_report=os.path.join(self.reference_folder, 'amr_summary.csv'),
                             test_report=os.path.join(self.test_folder, 'amr_summary.csv'),
                             columns_to_exclude=['Strain', 'Contig'],
                             identifying_column='Strain',
                             one_to_one=True)
        # cgMLST
        logging.info('Validating cgmlst.csv')
        self.validate_report(reference_report=os.path.join(self.reference_folder, 'cgmlst.csv'),
                             test_report=os.path.join(self.test_folder, 'cgmlst.csv'),
                             columns_to_exclude=['Strain'],
                             identifying_column='Strain',
                             one_to_one=True)
        # ConFindr
        logging.info('Validating confindr_report.csv')
        self.validate_report(reference_report=os.path.join(self.reference_folder, 'confindr_report.csv'),
                             test_report=os.path.join(self.test_folder, 'confindr_report.csv'),
                             columns_to_exclude=['Strain'],
                             identifying_column='Strain')
        # GDCS
        logging.info('Validating gdcs.csv')
        self.validate_report(reference_report=os.path.join(self.reference_folder, 'gdcs.csv'),
                             test_report=os.path.join(self.test_folder, 'gdcs.csv'),
                             columns_to_exclude=['Strain'],
                             identifying_column='Strain')
        # GeneSippr
        logging.info('Validating genesippr.csv')
        self.validate_report(reference_report=os.path.join(self.reference_folder, 'genesippr.csv'),
                             test_report=os.path.join(self.test_folder, 'genesippr.csv'),
                             columns_to_exclude=['Strain'],
                             identifying_column='Strain')
        # Legacy V-typer
        logging.info('Validating legacy_vtyper.csv')
        self.validate_report(reference_report=os.path.join(self.reference_folder, 'legacy_vtyper.csv'),
                             test_report=os.path.join(self.test_folder, 'legacy_vtyper.csv'),
                             columns_to_exclude=['Strain'],
                             identifying_column='Strain')
        # MASH
        logging.info('Validating mash.csv')
        self.validate_report(reference_report=os.path.join(self.reference_folder, 'mash.csv'),
                             test_report=os.path.join(self.test_folder, 'mash.csv'),
                             columns_to_exclude=['Strain', 'NumMatchingHashes', 'ReferenceGenomeMashDistance'],
                             identifying_column='Strain')
        # MLST Bacillus
        logging.info('Validating mlst_Bacillus.csv')
        self.validate_report(reference_report=os.path.join(self.reference_folder, 'mlst_Bacillus.csv'),
                             test_report=os.path.join(self.test_folder, 'mlst_Bacillus.csv'),
                             columns_to_exclude=['Strain'],
                             identifying_column='Strain',
                             one_to_one=True)
        # MLST Campylobacter
        logging.info('Validating mlst_Campylobacter.csv')
        self.validate_report(reference_report=os.path.join(self.reference_folder, 'mlst_Campylobacter.csv'),
                             test_report=os.path.join(self.test_folder, 'mlst_Campylobacter.csv'),
                             columns_to_exclude=['Strain'],
                             identifying_column='Strain',
                             one_to_one=True)
        # MLST Cronobacter
        logging.info('Validating mlst_Campylobacter.csv')
        self.validate_report(reference_report=os.path.join(self.reference_folder, 'mlst_Cronobacter.csv'),
                             test_report=os.path.join(self.test_folder, 'mlst_Cronobacter.csv'),
                             columns_to_exclude=['Strain'],
                             identifying_column='Strain',
                             one_to_one=True)
        # MLST Escherichia
        logging.info('Validating mlst_Escherichia.csv')
        self.validate_report(reference_report=os.path.join(self.reference_folder, 'mlst_Escherichia.csv'),
                             test_report=os.path.join(self.test_folder, 'mlst_Escherichia.csv'),
                             columns_to_exclude=['Strain'],
                             identifying_column='Strain',
                             one_to_one=True)
        # MLST Listeria
        logging.info('Validating mlst_Listeria.csv')
        self.validate_report(reference_report=os.path.join(self.reference_folder, 'mlst_Listeria.csv'),
                             test_report=os.path.join(self.test_folder, 'mlst_Listeria.csv'),
                             columns_to_exclude=['Strain'],
                             identifying_column='Strain',
                             one_to_one=True)
        # MLST Salmonella
        logging.info('Validating mlst_Salmonella.csv')
        self.validate_report(reference_report=os.path.join(self.reference_folder, 'mlst_Salmonella.csv'),
                             test_report=os.path.join(self.test_folder, 'mlst_Salmonella.csv'),
                             columns_to_exclude=['Strain'],
                             identifying_column='Strain',
                             one_to_one=True)
        # MLST Vibrio
        logging.info('Validating mlst_Vibrio.csv')
        self.validate_report(reference_report=os.path.join(self.reference_folder, 'mlst_Vibrio.csv'),
                             test_report=os.path.join(self.test_folder, 'mlst_Vibrio.csv'),
                             columns_to_exclude=['Strain'],
                             identifying_column='Strain',
                             one_to_one=True)
        # MOB-Recon
        # logging.info('Validating mob_recon_summary.csv')
        # self.validate_report(reference_report=os.path.join(self.reference_folder, 'mob_recon_summary.csv'),
        #                      test_report=os.path.join(self.test_folder, 'mob_recon_summary.csv'),
        #                      columns_to_exclude=['Strain', 'Contig', 'Incompatibility', 'IncompatibilityAccession',
        #                                          'RelaxaseType'],
        #                      identifying_column='Strain',
        #                      one_to_one=True,
        #                      check_rows=False,
        #                      resfinder=True)
        # Prophages
        logging.info('Validating prophages.csv')
        self.validate_report(reference_report=os.path.join(self.reference_folder, 'prophages.csv'),
                             test_report=os.path.join(self.test_folder, 'prophages.csv'),
                             columns_to_exclude=['Strain', 'Contig'],
                             identifying_column='Strain',
                             resfinder=True)
        # Run metrics report
        logging.info('Validating run_metrics_report.csv')
        self.validate_report(reference_report=os.path.join(self.reference_folder, 'run_metrics_report.csv'),
                             test_report=os.path.join(self.test_folder, 'run_metrics_report.csv'),
                             columns_to_exclude=[],
                             identifying_column='RunName')
        # ResFinder
        logging.info('Validating resfinder.csv')
        self.validate_report(reference_report=os.path.join(self.reference_folder, 'resfinder.csv'),
                             test_report=os.path.join(self.test_folder, 'resfinder.csv'),
                             columns_to_exclude=['Strain'],
                             identifying_column='Strain',
                             one_to_one=True)
        # rMLST
        logging.info('Validating rmlst.csv')
        self.validate_report(reference_report=os.path.join(self.reference_folder, 'rmlst.csv'),
                             test_report=os.path.join(self.test_folder, 'rmlst.csv'),
                             columns_to_exclude=['Strain'],
                             identifying_column='Strain',
                             one_to_one=True)
        # # rMLST Assembled
        # logging.info('Validating rmlst_assembled.csv')
        # self.validate_report(reference_report=os.path.join(self.reference_folder, 'rmlst_assembled.csv'),
        #                      test_report=os.path.join(self.test_folder, 'rmlst_assembled.csv'),
        #                      columns_to_exclude=['Strain'],
        #                      identifying_column='Strain',
        #                      one_to_one=True)
        # serosippr
        logging.info('Validating serosippr.csv')
        self.validate_report(reference_report=os.path.join(self.reference_folder, 'serosippr.csv'),
                             test_report=os.path.join(self.test_folder, 'serosippr.csv'),
                             columns_to_exclude=['Strain'],
                             identifying_column='Strain')
        # sistr
        logging.info('Validating sistr.tsv')
        self.validate_report(reference_report=os.path.join(self.reference_folder, 'sistr.tsv'),
                             test_report=os.path.join(self.test_folder, 'sistr.tsv'),
                             columns_to_exclude=['genome'],
                             identifying_column='genome',
                             separator='\t')
        # 16S
        logging.info('Validating sixteens_full.csv')
        self.validate_report(reference_report=os.path.join(self.reference_folder, 'sixteens_full.csv'),
                             test_report=os.path.join(self.test_folder, 'sixteens_full.csv'),
                             columns_to_exclude=['Strain', 'Gene'],
                             identifying_column='Strain')
        # Univec
        logging.info('Validating univec.csv')
        self.validate_report(reference_report=os.path.join(self.reference_folder, 'univec.csv'),
                             test_report=os.path.join(self.test_folder, 'univec.csv'),
                             columns_to_exclude=['Strain', 'Contig', 'Location'],
                             identifying_column='Strain',
                             resfinder=True)
        # Virulence
        logging.info('Validating virulence.csv')
        self.validate_report(reference_report=os.path.join(self.reference_folder, 'virulence.csv'),
                             test_report=os.path.join(self.test_folder, 'virulence.csv'),
                             columns_to_exclude=['Strain'],
                             identifying_column='Strain',
                             one_to_one=True)

    def validate_combined_metadata(self, reference_report, test_report, assembly_typer=False):
        column_list = list()
        column_list.append(validate.Column(name='Genus'))
        column_list.append(validate.Column(name='N50', column_type='Range', acceptable_range=40000))
        column_list.append(validate.Column(name='NumContigs', column_type='Range', acceptable_range=50))
        column_list.append(validate.Column(name='TotalLength', column_type='Range', acceptable_range=100000))
        column_list.append(validate.Column(name='PercentGC', column_type='Range', acceptable_range=2))
        column_list.append(validate.Column(name='rMLST_Result'))
        column_list.append(validate.Column(name='MLST_Result'))
        column_list.append(validate.Column(name='E_coli_Serotype'))
        column_list.append(validate.Column(name='SISTR_serovar_cgMLST'))
        column_list.append(validate.Column(name='GeneSeekr_Profile'))
        column_list.append(validate.Column(name='Vtyper_Profile'))
        column_list.append(validate.Column(name='AMR_Profile'))
        column_list.append(validate.Column(name='TotalPredictedGenes', column_type='Range', acceptable_range=50))
        if not assembly_typer:
            column_list.append(validate.Column(name='ConfindrContamSNVs'))
            column_list.append(validate.Column(name='MeanInsertSize', column_type='Range', acceptable_range=50))
            column_list.append(validate.Column(name='InsertSizeSTD', column_type='Range', acceptable_range=50))
            column_list.append(validate.Column(name='AverageCoverageDepth', column_type='Range', acceptable_range=10))
            column_list.append(validate.Column(name='CoverageDepthSTD', column_type='Range', acceptable_range=10))

        validator = validate.Validator(reference_csv=reference_report,
                                       test_csv=test_report,
                                       column_list=column_list,
                                       identifying_column='SeqID')

        validation_list = [validator.same_columns_in_ref_and_test(),
                           validator.all_test_columns_in_ref_and_test(),
                           validator.check_samples_present(),
                           validator.check_columns_match()]
        if False in validation_list:
            self.validate_pass = False
        else:
            self.validate_pass = True

    @staticmethod
    def validate_report(reference_report, test_report, columns_to_exclude, identifying_column, one_to_one=False,
                        resfinder=False, separator=',', check_rows=True):
        columns = validate.find_all_columns(csv_file=test_report,
                                            columns_to_exclude=columns_to_exclude,
                                            separator=separator)
        report_validate = validate.Validator(reference_csv=reference_report,
                                             test_csv=test_report,
                                             column_list=columns,
                                             identifying_column=identifying_column,
                                             separator=separator)
        if resfinder or one_to_one:
            if one_to_one:
                assert report_validate.check_resfinderesque_output(one_to_one=True, check_rows=check_rows) is True
            else:
                assert report_validate.check_resfinderesque_output(check_rows=check_rows) is True
        else:
            assert report_validate.all_test_columns_in_ref_and_test() is True
            assert report_validate.same_columns_in_ref_and_test() is True
            assert report_validate.check_samples_present() is True
            assert report_validate.check_columns_match() is True

    def __init__(self, reference_folder, test_folder, assembly_typer=False):
        """

        :param reference_folder:
        :param test_folder:
        :param assembly_typer:
        """
        if reference_folder.startswith('~'):
            self.reference_folder = os.path.abspath(os.path.expanduser(os.path.join(reference_folder)))
        else:
            self.reference_folder = os.path.abspath(os.path.join(reference_folder))
        assert os.path.isdir(self.reference_folder), 'Cannot locate reference report folder as specified: {rf}'\
            .format(rf=self.reference_folder)
        if test_folder.startswith('~'):
            self.test_folder = os.path.abspath(os.path.expanduser(os.path.join(test_folder)))
        else:
            self.test_folder = os.path.abspath(os.path.join(test_folder))
        assert os.path.isdir(self.test_folder), 'Cannot locate test report folder as specified: {tf}'\
            .format(tf=self.test_folder)
        self.assembly_typer = assembly_typer
        self.validate_pass = False


if __name__ == '__main__':
    # Parser for arguments
    parser = ArgumentParser(description='Run integration tests on COWBAT pipeline')
    parser.add_argument('-r', '--reference_folder',
                        required=True,
                        help='Path to reference folder with CSV reports with expected results.')
    parser.add_argument('-t', '--test_folder',
                        required=True,
                        help='Path to test folder with CSV reports with observed results .')
    parser.add_argument('-a', '--assembly',
                        action='store_true',
                        help='The assembly typing pipeline was used to process the run, rather than full COWBAT')
    # Get the arguments into an object
    args = parser.parse_args()
    # Pretty logging!
    SetupLogging()
    # Test the reports.
    validate_outputs = ValidateCowbat(reference_folder=args.reference_folder,
                                      test_folder=args.test_folder,
                                      assembly_typer=args.assembly)
    validate_outputs.validate_cowbat()
    if validate_outputs.validate_pass:
        logging.info('COWBAT successfully validated! :D')
    else:
        logging.error('COWBAT not successfully validated.')
